
import pytest

@pytest.fixture
def mapping_response():
    return {'responseHeader': {'QTime': 0, 'params': {'mm': '100%', 'q': '*:*', 'defType': 'edismax', 'facet_min_count': '1', 'hl': 'false', 'start': '0', 'q.op': 'AND', 'fq': 'subject_id:"MONDO\\:0020121" OR object_id:"MONDO\\:0020121"', 'facet.mincount': '1', 'rows': '20', 'facet': 'true'}}, 'response': {'num_found': 9, 'start': 0, 'docs': [{'subject_id': 'MONDO:0020121', 'predicate_id': 'skos:exactMatch', 'object_id': 'ICD10CM:G71.0', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'subject_label': 'muscular dystrophy', 'object_label': 'Muscular dystrophy', 'id': '6c90333f-7a4e-4c24-9ba8-b7b421f23f59', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'predicate_id': 'skos:exactMatch', 'object_id': 'Orphanet:98473', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'subject_label': 'muscular dystrophy', 'object_label': 'Muscular dystrophy', 'id': '8fee2808-f910-4123-8a36-fd5b54d3be4f', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'predicate_id': 'skos:exactMatch', 'object_id': 'NCIT:C84910', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'subject_label': 'muscular dystrophy', 'object_label': 'Muscular Dystrophy', 'id': '4013ddcb-e89f-427c-b02c-23418ef76d88', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'predicate_id': 'skos:exactMatch', 'object_id': 'DOID:9884', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'subject_label': 'muscular dystrophy', 'object_label': 'muscular dystrophy', 'id': '6844608d-6976-4b26-930f-d0721b4a9c80', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'predicate_id': 'skos:exactMatch', 'object_id': 'UMLS:C0026850', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'subject_label': 'muscular dystrophy', 'id': 'c65b8375-f30e-42a8-8fc7-70727f970dec', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'predicate_id': 'skos:exactMatch', 'object_id': 'MESH:D009136', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'subject_label': 'muscular dystrophy', 'id': 'f5e44c37-521e-42ee-8c3d-122ae2354201', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'predicate_id': 'skos:exactMatch', 'object_id': 'SCTID:73297009', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'subject_label': 'muscular dystrophy', 'id': 'f828cde8-ced0-486d-8bb7-73f43a1195c6', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'predicate_id': 'skos:exactMatch', 'object_id': 'MEDGEN:44527', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'subject_label': 'muscular dystrophy', 'id': '7a201e87-f5ef-43da-a8a6-8a015d278c92', 'highlighting': None}, {'subject_id': 'MONDO:0020121', 'predicate_id': 'skos:exactMatch', 'object_id': 'icd11.foundation:1464662404', 'mapping_justification': 'semapv:UnspecifiedMatching', 'mapping_source': ['mondo.sssom'], 'subject_label': 'muscular dystrophy', 'id': 'bb176d6d-27d6-4c54-8bb9-11b1e8e7fdab', 'highlighting': None}]}, 'facet_counts': {'facet_fields': {}, 'facet_queries': {}}, 'highlighting': {}}
